@@ -28,6 +28,15 @@ def pytest_addoption(parser):
     parser.addini(
         "flake8-max-line-length",
         help="maximum line length")
+    parser.addini(
+        "flake8-max-complexity",
+        help="McCabe complexity threshold")
+    parser.addini(
+        "flake8-show-source", type="bool",
+        help="show the source generate each error or warning")
+    parser.addini(
+        "flake8-statistics", type="bool",
+        help="count errors and warnings")
     parser.addini(
         "flake8-extensions", type="args", default=[".py"],
         help="a list of file extensions, for example: .py .pyx")
@@ -38,8 +47,12 @@ def pytest_configure(config):
     if config.option.flake8:
         config._flake8ignore = Ignorer(config.getini("flake8-ignore"))
         config._flake8maxlen = config.getini("flake8-max-line-length")
+        config._flake8maxcomplexity = config.getini("flake8-max-complexity")
+        config._flake8showshource = config.getini("flake8-show-source")
+        config._flake8statistics = config.getini("flake8-statistics")
         config._flake8exts = config.getini("flake8-extensions")
-        config._flake8mtimes = config.cache.get(HISTKEY, {})
+        if hasattr(config, 'cache'):
+            config._flake8mtimes = config.cache.get(HISTKEY, {})
 
 
 def pytest_collect_file(path, parent):
@@ -48,7 +61,14 @@ def pytest_collect_file(path, parent):
     if config.option.flake8 and path.ext in config._flake8exts:
         flake8ignore = config._flake8ignore(path)
         if flake8ignore is not None:
-            return Flake8Item(path, parent, flake8ignore, config._flake8maxlen)
+            return Flake8Item(
+                path,
+                parent,
+                flake8ignore=flake8ignore,
+                maxlength=config._flake8maxlen,
+                maxcomplexity=config._flake8maxcomplexity,
+                showshource=config._flake8showshource,
+                statistics=config._flake8statistics)
 
 
 def pytest_unconfigure(config):
@@ -63,14 +83,21 @@ class Flake8Error(Exception):
 
 class Flake8Item(pytest.Item, pytest.File):
 
-    def __init__(self, path, parent, flake8ignore, maxlength):
+    def __init__(self, path, parent, flake8ignore, maxlength,
+                 maxcomplexity, showshource, statistics):
         super(Flake8Item, self).__init__(path, parent)
         self.add_marker("flake8")
         self.flake8ignore = flake8ignore
         self.maxlength = maxlength
+        self.maxcomplexity = maxcomplexity
+        self.showshource = showshource
+        self.statistics = statistics
 
     def setup(self):
-        flake8mtimes = self.config._flake8mtimes
+        if hasattr(self.config, "_flake8mtimes"):
+            flake8mtimes = self.config._flake8mtimes
+        else:
+            flake8mtimes = {}
         self._flake8mtime = self.fspath.mtime()
         old = flake8mtimes.get(str(self.fspath), (0, []))
         if old == [self._flake8mtime, self.flake8ignore]:
@@ -79,13 +106,20 @@ class Flake8Item(pytest.Item, pytest.File):
     def runtest(self):
         call = py.io.StdCapture.call
         found_errors, out, err = call(
-            check_file, self.fspath, self.flake8ignore, self.maxlength)
+            check_file,
+            self.fspath,
+            self.flake8ignore,
+            self.maxlength,
+            self.maxcomplexity,
+            self.showshource,
+            self.statistics)
         if found_errors:
             raise Flake8Error(out, err)
         # update mtime only if test passed
         # otherwise failures would not be re-run next time
-        self.config._flake8mtimes[str(self.fspath)] = (self._flake8mtime,
-                                                       self.flake8ignore)
+        if hasattr(self.config, "_flake8mtimes"):
+            self.config._flake8mtimes[str(self.fspath)] = (self._flake8mtime,
+                                                           self.flake8ignore)
 
     def repr_failure(self, excinfo):
         if excinfo.errisinstance(Flake8Error):
@@ -130,11 +164,18 @@ class Ignorer:
         return l
 
 
-def check_file(path, flake8ignore, maxlength):
+def check_file(path, flake8ignore, maxlength, maxcomplexity,
+               showshource, statistics):
     """Run flake8 over a single file, and return the number of failures."""
     args = []
     if maxlength:
-        args = ['--max-line-length', maxlength]
+        args += ['--max-line-length', maxlength]
+    if maxcomplexity:
+        args += ['--max-complexity', maxcomplexity]
+    if showshource:
+        args += ['--show-source']
+    if statistics:
+        args += ['--statistics']
     app = application.Application()
     app.find_plugins()
     app.register_plugin_options()
